#' The application server-side
#'
#' @param input,output,session Internal parameters for {shiny}.
#'     DO NOT REMOVE.
#' @import shiny
#' @import analycyte.projects
#' @import shinyFiles
#' @importFrom shinyjs runjs
#' @import fs
#' @import dplyr
#' @import utils
#' @import shinydashboard
#' @importFrom shinycssloaders withSpinner
#' @importFrom shinyWidgets actionBttn pickerInput
#' @noRd
app_server <- function(input, output, session) {
  # Initialize values and reactive elements
  # if(Sys.getenv('SHINY_PORT') != "")
  options(shiny.maxRequestSize = 500 * 1024^2)

  old_home <- Sys.getenv("HOME")
  old_ppath <- Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")
  r_global <- reactiveValues()
  reloadTrigger <- reactiveVal(0)
  project_update <- reactiveVal(0)
  allTabs <- reactiveVal()

  projectExists <- reactiveVal(FALSE)
  utils::data("tools_ex")

  observe({
    is_empty <- length(allTabs()) == 0 # Check if the navbar is empty

    if (!projectExists()) {
      shinyjs::hide("show_tool_box")
    } else {
      shinyjs::show("show_tool_box")
    }

    if (is_empty) {
      shinyjs::hide("myTabs") # Hide or show based on whether it's empty
    } else {
      shinyjs::show("myTabs")
    }
  })


  menu_params <- list(
    list(
      id = "tab_dynamic", title = "Expert View", tabname = NULL, iconname = "table",
      subitems = list(
        list(subtitle = "Project List", subtabname = "project", subiconname = "folder-open"),
        list(subtitle = "User Actions", subtabname = "history", subiconname = "clock-rotate-left"),
        list(subtitle = "Users Info", subtabname = "authors_affiliations", subiconname = "users"),
        list(subtitle = "Available Modules", subtabname = "tools", subiconname = "wrench")
      )
    ),
    list(id = "data_dynamic", title = "Dataset", tabname = "in_data", iconname = "database", subitems = NULL)
  )

  lapply(menu_params, function(params) {
    output[[params$id]] <- renderUI({
      if (is.null(params$subitems)) {
        menuItem(params$title, tabName = params$tabname, icon = icon(params$iconname))
      } else {
        # When there are subitems, we'll create a list of subitems to nest under the main item
        subitems_list <- lapply(params$subitems, function(sub) {
          menuSubItem(sub$subtitle, tabName = sub$subtabname, icon = icon(sub$subiconname))
        })
        # Return the main item with nested subitems
        do.call(menuItem, c(list(params$title, icon = icon(params$iconname)), subitems_list))
      }
    })
  })

  lapply(menu_params, function(params) {
    observe({
      shinyjs::toggleElement(
        id = params$id,
        condition = projectExists()
      )
    })
  })

  observe({
    if (!is.null(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")) && Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH") != "") {
      projectExists(TRUE)
      print("TRUE")
    } else {
      projectExists(FALSE)
      print("FALSE")
    }
  })

  observe({
    if (!dir.exists(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH"))) {
      projectExists(FALSE)
      Sys.unsetenv("ANALYCYTE_PROJECTS_FOLDER_PATH")
    }
  })

  observe({
    cat("Setting input.projectExists to:", projectExists(), "\n")
    if (projectExists()) {
      shinyjs::runjs("Shiny.setInputValue('projectExists', true);")
    } else {
      shinyjs::runjs("Shiny.setInputValue('projectExists', false);")
    }
  })

  # Define a reactive expression for the environment values
  envValues <- reactive({
    reloadTrigger()

    list(div(
      p("HOME: ", Sys.getenv("HOME")),
      p(
        "prod: ",
        if (is.null(options()$golem.app.prod)) {
          tags$span(
            options("golem.app.prod"),
            inline_notification_item("no Dev or Prod mod detected", class = "fa-solid fa-code", color = "orange")
          )
        } else {
          options("golem.app.prod")
        }
      ),
      p(
        "ANALYCYTE_PROJECTS_FOLDER_PATH: ",
        if (is.null(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")) || Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH") == "") {
          tags$span(
            Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH"),
            inline_notification_item("Need to setup a project", class = "fa-solid fa-triangle-exclamation", color = "red")
          )
        } else {
          Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")
        }
      ),
      p("SHINY_PORT: ", Sys.getenv("SHINY_PORT")),
      p("SHINY_SERVER_VERSION: ", Sys.getenv("SHINY_SERVER_VERSION"))
    ))
  })

  output$env_values <- renderUI({
    envValues()
  })


  output$env_export_project <- renderUI({
    req(projectExists())
    req(input$selectedProject)
    req(r_global$project_id)
    reloadTrigger()
    downloadButton("downloadProjectButton", paste("Download Project ", input$selectedProject),
      class = "btn btn-primary edit_btn white pull-right"
    ) # Use Bootstrap classes for styling)
  })

  output$downloadProjectButton <- downloadHandler(
    filename = function() {
      # Define the filename, e.g., "project.zip"
      paste("analycyte-", input$selectedProject, "-", Sys.Date(), ".zip", sep = "")
    },
    content = function(file) {
      # Call your export_project function here
      # Save the output to the file path provided by Shiny
      export_project(project = r_global$project_id, zipfile = file, include_hidden = FALSE, exclude = NULL)
    }
  )


  volumes <- c(
    Home = old_home, "R Installation" = R.home(),
    shinyFiles::getVolumes()()
  )

  shinyDirChoose(input, "directory",
    roots = volumes,
    restrictions = system.file(package = "base"),
    allowDirCreate = TRUE,
    defaultRoot = "Home",
    session = session
  )
  dirDown <- reactive(input$directory)

  observeEvent(
    ignoreNULL = TRUE,
    eventExpr = {
      input$directory
    },
    handlerExpr = {
      if (!"path" %in% names(dirDown())) {
        return()
      }
      home <- normalizePath("~")
      r_global$datapath <-
        file.path(home, paste(unlist(dirDown()$path[-1]), collapse = .Platform$file.sep))


      reloadTrigger(reloadTrigger() + 1)
    }
  )

  observeEvent(r_global$datapath, {
    if (all(paste(r_global$datapath, c("projects", ".metadata", ".templates"), sep = "/") %in% fs::dir_ls(r_global$datapath, all = TRUE))) {
      Sys.setenv(HOME = r_global$datapath, ANALYCYTE_PROJECTS_FOLDER_PATH = r_global$datapath)
      projectExists(TRUE)
    } else {
      Sys.setenv(HOME = r_global$datapath)
      setup_projects(path = r_global$datapath)
      populate_tools(tools_ex)
      projectExists(TRUE)
    }
  })

  # Project setup logic
  output$projectSetupUI <- renderUI({
    reloadTrigger()

    if (Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH") == "") {
      div(
        align = "center",
        shinyFiles::shinyDirButton("directory", class = "btn btn-primary white", "Select folder", "Please select a folder")
        # , actionButton("setupTemp", class = "btn-dir", "Temporary Directory")
      )
    } else {
      div(
        align = "center",
        actionButton("unsyncProject", class = "btn btn-primary edit_btn white", "Close Session"),
        actionButton("deleteProject", class = "btn btn-danger  delete_btn white", "Delete Session")
      )
    }
  })

  # observeEvent(input$setupTemp, {
  #   temp_dir <- tempfile("dir")
  #   dir.create(temp_dir)
  #   # Ensure ANALYCYTE_PROJECTS_FOLDER_PATH is unset
  #   Sys.unsetenv("ANALYCYTE_PROJECTS_FOLDER_PATH")
  #
  #   Sys.setenv(HOME = temp_dir)
  #   setup_projects(path = temp_dir)
  #   populate_tools(tools_ex)
  #
  #   projectExists(TRUE)
  # })

  resetProject <- function(deleteFiles = FALSE) {
    if (deleteFiles) {
      unlink(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH"), recursive = TRUE)
    }
    Sys.unsetenv("ANALYCYTE_PROJECTS_FOLDER_PATH")
    Sys.setenv(HOME = old_home)
    removeModal()
    projectExists(FALSE)
    tabs_to_remove <- allTabs()
    for (tab in tabs_to_remove) {
      removeTab(inputId = "myTabs", target = tab)
    }
    allTabs(c())
  }

  observeEvent(input$deleteProject, {
    # Show confirmation modal when "Delete Project" button is clicked
    showModal(
      modalDialog(
        type = "error",
        title = div("Are you sure?",class="my-modal"),
        "Do you really want to delete the project?",
        br(), "The folder in",
        Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH"),
        "will be deleted, this process cannot be undone.",
        footer = tagList(
          # modalButton("Cancel"),
          tags$button(
            type = "button", class = "btn btn-sm pull-left",
            `data-dismiss` = "modal", `data-bs-dismiss` = "modal",
            "Cancel"
          ),
          actionButton("confirm_delete", "Yes, Delete", class = "btn btn-danger btn-sm delete_btn white")
        )
      )
    )
  })

  observeEvent(input$confirm_delete, {
    resetProject(deleteFiles = TRUE)
    reloadTrigger(reloadTrigger() + 1)
    r_global <<- reactiveValues() # use the global assignment operator <<-
  })

  observeEvent(input$unsyncProject, {
    resetProject(deleteFiles = FALSE)
    reloadTrigger(reloadTrigger() + 1)
  })

  # Session ending logic
  session$onSessionEnded(function() {
    Sys.setenv(HOME = old_home)
  })

  # Reload trigger logic
  observeEvent(input$reload, {
    reloadTrigger(reloadTrigger() + 1)
    project_update(project_update() + 1)
  })

  load_Steps <- reactive({
    req(projectExists())
    reloadTrigger()
    steps()
  })

  output$steps_table <- renderUI({
    withSpinner(DT::dataTableOutput("show_steps_history"), type=6)
  })

  output$show_steps_history <- DT::renderDataTable(
    {
      # Check if input$selectedProject is undetermined
      if (is.null(input$selectedProject) || input$selectedProject == "") {
       # return( DT::datatable(load_Steps(), options = list(pageLength = -1),
       #                       rownames = FALSE) )
        # return(load_Steps())
        return(load_Steps())

      }

   # DT::datatable(  
     Project_Steps() %>%
        dplyr::filter(PID == r_global$project_id) %>% # dplyr::filter(PID == gsub("p|0", "", input$selectedProject)) %>%
        dplyr::select(c("SID", "PID", "hash_tag")) 
        # , options = list(pageLength = -1),
        # rownames = FALSE)
    },
    # {
    #   Project_Steps()  
    # },
    rownames = FALSE
  )

  load_data <- function(data_function) {
    req(projectExists())
    reloadTrigger()
    data_function()
  }

  load_Projects <- reactive({
    load_data(projects)
  })
  load_Autho <- reactive({
    load_data(authors)
  })
  load_Tools <- reactive({
    load_data(tools)
  })
  load_Aff <- reactive({
    load_data(affiliations)
  })

  observe({
    withSpinner(mod_table_server(id = "project_table", table = load_Projects(), what = "project", ID = "id", l=-1) , type = 6)
    withSpinner(  mod_table_server(id = "author_table", table = load_Autho(), what = "author", ID = "id"), type = 6)
    withSpinner(   mod_table_server(id = "affilation_table", table = load_Aff(), what = "affiliation", ID = "id"), type = 6)
    withSpinner(   mod_table_server(id = "large_steps_table", table = Project_Steps(), what = "step", ID = "SID", l=-1), type = 6)

    # datamods::edit_data_server("author_bis",  data_r = load_Autho )
    # datamods::edit_data_server("large_steps_table_bis",  data_r = load_Steps )
  })

  observeEvent(input$`project_table-add`, {
    mod_new_row_server(id = "project_table", what = "project", reloadTrigger, project_update)
  })

  observeEvent(input$`author_table-add`, {
    mod_new_row_server(id = "author_table", what = "author", reloadTrigger, project_update)
  })

  observeEvent(input$`affilation_table-add`, {
    mod_new_row_server(id = "affiliation_table", what = "affiliation", reloadTrigger, project_update)
  })

  output$tool_table <- renderUI({
    withSpinner(DT::dataTableOutput("show_tool"), type = 6)
  })

  output$show_tool <- DT::renderDataTable(load_Tools(),
    options = list(scrollX = TRUE),
    rownames = FALSE
  )
  observe({
    if (projectExists()) {
      if (nrow(load_Projects()) == 0 || nrow(load_Tools()) == 0) { # Adjust the logic as per your requirements
        showModal(modalDialog(
          title = div("Getting Started Guide", class="modal-title"),
          "To start working, it's essential to have available tools and at least one project.",
          if (nrow(load_Projects()) == 0) {
            tags$div(
              "It seems you don't have any projects. Would you like to create a new one?"
            )
          } else {
            tags$div(
              "Please add tools in the tools table to proceed."
            )
          },
          easyClose = FALSE # To ensure user interacts with the modal
          ,        footer = tagList(
            # modalButton("Cancel"),
            tags$button(
              type = "button", class = "btn btn-sm pull-left",
              `data-dismiss` = "modal", `data-bs-dismiss` = "modal",
              "Cancel"
            ),
            if (nrow(load_Projects()) == 0) {
            actionButton("mockButton", "Create a new project", class = "btn btn-primary btn-sm edit_btn white")},
            if (nrow(load_Tools()) == 0) {
            actionButton("mockButton2", "Populate tools table with a template", class = "btn btn-primary btn-sm edit_btn white")}
          )
        ))
      }
    }
  })

  observeEvent(input$mockButton, {
    # Créer un nouveau projet avec la fonction new_project()
    mod_new_row_server(id = "first_project_table", what = "project", reloadTrigger, project_update)
    # reloadTrigger(reloadTrigger() + 1)
    # project_update(project_update() + 1)
  })




  observeEvent(input$mockButton2, {
    reloadTrigger(reloadTrigger() + 1)
    populate_tools(tools_ex)
    removeModal()
  })

  output$projectSelect <- renderUI({
    req(projectExists())
    req(dir.exists(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")))

    project_folders <- fs::dir_ls(path = Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH"), type = "directory")

    # Extract project IDs from folder names (assuming folder names have format P0000<id>)
    project_names <- gsub(".*/", "", c(project_folders)) |> unname()

    pickerInput("selectedProject", "Choose a Project:", choices = project_names)
  })

  observe({
    project_update()
    project_folder_path <- Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")

    if (dir.exists(project_folder_path)) {
      project_folders <- fs::dir_ls(path = project_folder_path, type = "directory")
      project_names <- gsub(".*/", "", c(project_folders)) |> unname()
      shinyWidgets::updatePickerInput(session, "selectedProject", choices = project_names, selected = input$selectedProject)
      shinyWidgets::updatePickerInput(session, "selectedFE",
        selected = input$selectedFE
      )
    }
  })

  observe({
    req(input$selectedProject)
    r_global$project_folder_name <- input$selectedProject

    # need to go through database catch the project id
    p_id <- projects(verbose = T) %>%
      dplyr::filter(grepl(r_global$project_folder_name, path)) %>%
      dplyr::pull(id)

    r_global$project_id <- p_id
  })

  Project_Steps <- reactive({
    req(load_Steps())
    req(r_global$project_id)

    load_Steps() %>%
      subset(PID == r_global$project_id)
  })

  output$feSelect <- renderUI({
    req(projectExists())
    req(dir.exists(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")))
    req(Project_Steps())
    project_update() # Force une dépendance sur project_update

    hash_names <- Project_Steps() %>%
      subset(output_type == "5: fe") %>%
      dplyr::pull("hash_tag")
    # Create a pickerInput for the projects
    # pickerInput("selectedFE", "Reference FE:", choices = c(hash_names))
    pickerInput("selectedFE", "Feature Set:", choices = c(hash_names))
  })

  observe({
    r_global$dataFE <- input$selectedFE
  })



  observe({
    # Ensure these are set before proceeding
    req(projectExists(), r_global$dataFE, dir.exists(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")))
    print(r_global$dataFE)
    # Construct your path
    potential_path <- file.path(projects_folder(), r_global$project_folder_name, r_global$dataFE, "dataFE.rds")

    # Check if the file exists
    if (file.exists(potential_path)) {
      print(potential_path)

      # Set the global variable and read in the data
      r_global$path_to_data <- file.path(projects_folder(), r_global$project_folder_name, r_global$dataFE)
      r_global$data <- readRDS(potential_path)
    } else {
      # Optional: Handle scenarios when the file does not exist
      message("Data file not found: ", potential_path)
    }

    # Trigger other reactive events if needed
    reloadTrigger()
  })


  ### sce
  output$sceSelect <- renderUI({
    req(projectExists())
    req(dir.exists(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")))
    req(Project_Steps())
    project_update() # Force une dépendance sur project_update

    hash_names <- Project_Steps() %>%
      subset(output_type == "3: sce") %>%
      dplyr::pull("hash_tag")
    # Create a pickerInput for the projects
    # pickerInput("selectedSCE", "Reference SCE:", choices = c(hash_names))
    pickerInput("selectedSCE", "Cell Set:",
      choices = c(hash_names), selected = NULL,
      multiple = TRUE,
      options = pickerOptions(maxOptions = 1)
    )
  })

  observe({
    r_global$dataSCE <- input$selectedSCE
  })



  observe({
    # Ensure these are set before proceeding
    req(projectExists(), r_global$dataSCE, dir.exists(Sys.getenv("ANALYCYTE_PROJECTS_FOLDER_PATH")))
    print(r_global$dataSCE)
    # Construct your path
    potential_path_SCE <- file.path(projects_folder(), r_global$project_folder_name, r_global$dataSCE, "dataSCE.rds")

    # Check if the file exists
    if (file.exists(potential_path_SCE)) {
      print(potential_path_SCE)

      # Set the global variable and read in the data
      r_global$path_to_data_sce <- file.path(projects_folder(), r_global$project_folder_name, r_global$dataSCE)
      r_global$data_sce <- readRDS(potential_path_SCE)
    } else {
      # Optional: Handle scenarios when the file does not exist
      message("Data file not found: ", potential_path_SCE)
    }

    # Trigger other reactive events if needed
    reloadTrigger()
  })
  ###

  output$toolSelect <- renderUI({
    req(Project_Steps())

    req(length(Project_Steps()) > 0)

    output_types <- unique(Project_Steps()[["output_type"]])

    possible_tools <- tools() %>%
      filter(prerequite_type %in% output_types)

    pickerInput("selectedTool", "Select an action:", choices = c("", possible_tools$description), selected = "", options = list(
      `live-search` = TRUE
    ))
  })

  counter_modules <- reactiveVal(1) # starting the counter from 1

  trick <- reactiveVal(0)
  res <- list()
  obs_close <- list()

  # Function to add a cross after tabPanel title
  tabTitle <- function(name, id) {
    tags$span(
      name, HTML("&nbsp;"),
      tags$span(
        id = paste0("close", id),
        class = "close",
        HTML("&times;")
      )
    )
  }

  closeTab <- function(id, allTabs) {
    # Remove tab from UI
    removeTab(inputId = "myTabs", target = paste(id))

    actual_tabs <- setdiff(allTabs(), id)

    allTabs(actual_tabs)
  }

  observeEvent(input$selectedTool, {
    # Check if a tool is selected
    if (is.null(input$selectedTool)) {
      # Send error message
      validate(
        need(FALSE, "Please select a tool before proceeding.")
      )
      return() # Important to exit early from the observer
    }

    module_name <- tools() %>%
      dplyr::filter(description == input$selectedTool) %>%
      pull(R_filename)

    current_id <- paste0(module_name, counter_modules())

    module_name_ui <- paste(module_name, "ui", sep = "_")
    module_name_server <- paste(module_name, "server", sep = "_")

    # Check if the functions exist in the modules list
    if (!is.null(modules[[module_name_ui]]) && !is.null(modules[[module_name_server]])) {
      fct_ui <- modules[[module_name_ui]]
      fct_server <- modules[[module_name_server]]
    } else if (input$selectedTool == "") {
      return(NULL)
    } else {
      shiny::showNotification(paste("Module", module_name, "is not implemented yet."), type = "error")
      message("Module ", module_name, " is not implemented yet.")
      return(NULL)
    }

    # Update allTabs to include the new tab
    current_tabs <- allTabs()
    allTabs(c(current_tabs, current_id))
    visual_name <- associated_name[[module_name]]
    local({
      if (module_name %in% c("mod_da", "mod_ds")) {
        # Module Server Logic
        fct_server(id = current_id, r_global = r_global, reloadTrigger, type = module_name)

        # UI Logic
        new_tab_content <- fct_ui(current_id, r_global = r_global, type = module_name)
      } else {
        # Module Server Logic
        fct_server(id = current_id, r_global = r_global, reloadTrigger)

        # UI Logic
        new_tab_content <- fct_ui(current_id, r_global = r_global)
      }


      # Append tab to the tabsetPanel
      appendTab(
        inputId = "myTabs",
        tabPanel(
          title = tabTitle(visual_name, current_id),
          value = current_id,
          tags$br(),
          new_tab_content
        ),
        select = TRUE
      )

      obs_close[[paste(current_id)]] <<- observe({
        shinyjs::onclick(id = paste0("close", current_id), closeTab(id = current_id, allTabs))
      })
    })
    counter_modules(counter_modules() + 1) # update the counter
  })


  observe({
    req(input$selectedFE)
    mod_dataset_server("current_fe", r_global, reloadTrigger)
  })



  observeEvent(input$close_app, {
    # If hosted on Shiny Server or RConnect, you can use:
    # stopApp()
    session$sendCustomMessage(type = "closeTab", message = "close") # For a local app, it will just stop the running instance without closing the browser window.
  })

  observe({
    session$sendCustomMessage(type = "toggleInTablesMenu", message = projectExists())
  })
}
