#' Create the reproducible HTML report
#'
#' @param output_dir Character. Path of the folder to save the report.
#' @param format format : choose between html, pdf, docx, revealjs, beamer, pptx
#' @param extension template extension : crcm
#' @param o_report precision of name
#' @param withProgress test to show a progress bar throught shiny
#' @param hash_tag Character. Path of the folder containing the report template.
#' @param list_param_report list of parameter, will be store in quarto-params yml
#'
#' @importFrom withr with_dir
#' @importFrom checkmate assertChoice assert
#' @importFrom quarto quarto_render
#' @importFrom stringr str_remove_all
#'
#' @return A list with the folder and the file of the created report.
#'
#' @export
create_report <- function(
    list_param_report,
    output_dir, # expect projects/p0001
    hash_tag, # expect an hash_tag
    format = "html", extension = "crcm",
    withProgress = NULL, 
    o_report = NULL
    # ,report_template = NULL # expect projects/p0001/progs
    ) {
  # checkmate::assertChoice(format,c("html", "pdf", "docx", "revealjs", "beamer", "pptx"))
  checkmate::assertChoice(format, c("html"))
  checkmate::assert(extension %in% c("crcm") || is.null(extension))

  # Checks on the list of parameters

  # list_param_report

  # Get the name of the html file
  if (is.null(o_report)) {
    output_report <- paste0(
      gsub("[.].*$", "", stringr::str_remove_all(Sys.time(), "-|:| |UTC")),
      "-analyticyte_report.", "html"
      # {if(format=="revealjs") "html" else if(format=="beamer") "pdf" else format}
    )
  } else {
    output_report <- paste0(
      gsub("[.].*$", "", stringr::str_remove_all(Sys.time(), "-|:| |UTC")),
      "-",
      o_report,
      ".",
      "html"
      # {if(format=="revealjs") "html" else if(format=="beamer") "pdf" else format}
    )
  }

  # Determine the Temporary Directory
  tmp_dir <- tempdir()

  # Move the .qmd template into the temp directory
  # fs::dir_copy(
  #   path = report_template,
  #   new_path = output_dir
  # )

  # Render the report
  withr::with_dir(
    file.path(output_dir, "progs"),
    {
      quarto::quarto_render(
        input = "Parent.qmd",
        execute_params = list_param_report,
        output_file = output_report,
        quiet = FALSE,
        output_format = {
          if (!is.null(extension)) paste(extension, format, sep = "-") else format
        }
      )
    }
  )

  fs::file_move(path = file.path(output_dir, "progs", output_report), new_path = file.path(output_dir, hash_tag, output_report))

  yaml_files <- list.files(path = tmp_dir, pattern = "\\.yml$", full.names = TRUE)
  target_yaml <- grep("quarto-params", yaml_files, value = TRUE)
  most_recent_yaml <- target_yaml[which.max(file.info(target_yaml)$mtime)] |> basename()
  renamed_yaml <- paste0(gsub("-analyticyte_report.html", "", output_report), "_params.yml")


  fs::file_move(path = file.path(tmp_dir, most_recent_yaml), new_path = file.path(output_dir, hash_tag, renamed_yaml))

  out_path <- list(
    output_dir_report = file.path(output_dir, hash_tag, output_report),
    output_file_report = output_report,
    output_yaml = renamed_yaml
  )


  add_resource_path(
    "reportlibrary", dirname(out_path$output_dir_report),
  )


  shinyjs::runjs(sprintf("window.open('%s', '_blank');", paste0("reportlibrary/", out_path$output_file_report)))


  return(out_path)
}
