#' @title Edit input form
#'
#' @description The `edit_input_form_2` function allows to correctly generate the variables to be edited in the modal window according to their respective class
#'
#' @param default default row on which to operate a modification or a deletion, otherwise empty list for an addition
#' @param var_mandatory vector of `character` which allows to choose obligatory fields to fill
#' @param selector list of params with type
#' @param session The `session` object passed to function given to shinyServer
#'
#' @importFrom shiny numericInput textInput
#' @importFrom shinyWidgets virtualSelectInput prettyCheckbox airDatepickerInput
#' @importFrom rlang  %||%
#'
#' @return different shiny widgets with edited columns according to their respective class
#' @noRd
#'
edit_input_form_2 <- function(default = list(),
                              selector,
                              var_mandatory,
                              session = getDefaultReactiveDomain()) {
  ns <- session$ns

  tagList(
    column(
      width = 12,
      lapply(
        X = seq_len(length(selector)),
        FUN = function(i) {
          variable_id <- names(selector[i])
          variable_name <- gsub("_", " ", names(selector[i]))
          variable <- selector[[i]]

          suffix <- if (isTRUE((inherits(variable, "logical")))) "" else " : "
          if (variable_name %in% var_mandatory) {
            label <- tagList(
              variable_name,
              tags$span(HTML("&#42;"), class = "asterisk", style = "color: red;"), suffix
            )
          } else {
            label <- paste0(variable_name, suffix)
          }

          if (isTRUE(inherits(variable, c("numeric", "integer")))) {
            numericInput(
              inputId = ns(variable_id),
              label = label,
              value = default[[variable_id]] %||% NA_real_,
              width = "100%"
            )
          } else if (isTRUE((inherits(variable, "factor")))) {
            virtualSelectInput(
              inputId = ns(variable_id),
              label = label,
              choices = sort(unique(c(as.character(variable), levels(variable)))),
              selected = default[[variable_id]] %||% "",
              width = "100%",
              allowNewOption = TRUE,
              autoSelectFirstOption = FALSE,
              placeholder = datamods::i18n("Select")
            )
          } else if (isTRUE((inherits(variable, "character")))) {
            textInput(
              inputId = ns(variable_id),
              label = label,
              value = default[[variable_id]] %||% "",
              width = "100%"
            )
          } else if (isTRUE((inherits(variable, "logical")))) {
            prettyCheckbox(
              inputId = ns(variable_id),
              label = label,
              value = default[[variable_id]] %||% FALSE,
              icon = icon("check"),
              status = "primary",
              width = "100%"
            )
          } else if (isTRUE((inherits(variable, "Date")))) {
            airDatepickerInput(
              inputId = ns(variable_id),
              label = label,
              value = default[[variable_id]] %||% Sys.Date(),
              inline = TRUE,
              width = "100%"
            )
          } else if (isTRUE((inherits(variable, c("POSIXct", "POSIXt"))))) {
            airDatepickerInput(
              inputId = ns(variable_id),
              label = label,
              value = default[[variable_id]] %||% Sys.time(),
              inline = TRUE,
              timepicker = TRUE,
              width = "100%"
            )
          } else if (isTRUE((inherits(variable, c("projects_author", "projects_stage", "steps_filetype"))))) {
            textInput(
              inputId = ns(variable_id),
              label = label,
              value = default[[variable_id]] %||% "",
              width = "100%"
            )
          } else {
            return(NULL)
          }
        }
      )
    )
  )
}
