#' import_concat UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @importFrom shiny NS tagList
#' @importFrom shinycssloaders withSpinner
mod_import_concat_ui <- function(id, r_global) {
  ns <- NS(id)
  tagList(
    fluidRow(
      box(
        width = 12, status = "primary",
        div(
          column(
            width = 6, strong("Project of Reference: "),
            span(style = "color: #007bff;", r_global$project_folder_name) # Styling for the text
          )
        )
      )
    ),
    fluidRow(
      column(
        width = 5,
        h2("FCS file import", class = "module-title-bold")
      ),
      column(width = 4),
      column(
        width = 3, br(),
        uiOutput(ns("createSCE_button")),
        br()
      )
    ),
    fluidRow(box(
      status = "warning", solidHeader = T, collapsible = T, collapsed = F,
      title = "Import from Concat", width = 12,
      markdown("
**Functionality Overview:**
This panel allows users to import a concatenated Flow Cytometry Standard (FCS) file representing a complete experiment. Each FCS file within the concatenated file should be annotated and assigned to specific biological samples, studied conditions, etc. Additionally, users must provide a metadata file (in `.xlsx` or `.csv` format) containing an FCS ID to identify the origin of cells.

**Instructions:**
1. **Uploading Files:**
   - **If using this application on a server**, first upload your files.
   - Click on the 'Browse' button (max file size: 500MB) to select and upload your concatenated FCS file and metadata file.
2. **Selecting FCS Directory:**
   - After uploading, select the 'uploaded_fcs' directory containing the fcs file by clicking on 'Select your FCS directory'.
3. **Metadata File Upload:**
   - Upload the metadata file that includes the FCS IDs. This file should be in either `.xlsx` or `.csv` format.
   - Ensure that the metadata file correctly maps each FCS ID to its respective sample and condition.

**File Requirements:**
- Concatenated FCS file must include properly annotated data with clustering and origine fcs id of each cells.
- Metadata file in `.xlsx` or `.csv` format, containing FCS IDs for cell origin identification.


**Marker Selection:**
- Once the concatenated FCS is loaded, manually select the marker of interest by clicking in the table.
               ")
    )),
    br(),
    fluidRow(
      box(
        title = span(icon("clipboard-list"), "Fcs contatened file"),
        status = "primary",
        width = 7,
        solidHeader = FALSE,
        column(
          width = 12,
          fileInput(ns("fileUploadonServer"), "Upload .fcs File", accept = c(".fcs")),
          shinyDirButton(ns("FCSdir"), "Select your FCS directory", "Load")
        ),
        column(
          width = 8,
          title = "Control Table", # width = 4,
          withSpinner( DT::dataTableOutput(outputId = ns("pData")), type =6)
        ),
        column(
          width = 4,
          title = "Control Table", # width = 3,
          # uiOutput(ns("ref_pData_name")),
          uiOutput(ns("Label_Markers_table_selected")),
          br(),
          uiOutput(ns("sharedColFcs_UI")),
          uiOutput(ns("Label_Clusters")),
          # fluidRow(
          #   column(
          #     width = 6,
          #     div(style = "width: 75px;", uiOutput(ns("transformation_fcs")))
          #   ),
          #   column(
          #     width = 6,
          #     div(style = "width: 100px;", uiOutput(ns("asinh_UI_cofact_fcs")))
          #   )
          # ),
          
          fluidRow(
            column( width = 12,
            uiOutput(ns("transformation_fcs")),
            uiOutput(ns("asinh_UI_cofact_fcs"))
            )
          )
        )
      ),
      box(
        title = span(icon("clipboard-list"), "Metadata Fcs table"),
        status = "primary",
        width = 5,


        # status = 'warning',
        solidHeader = FALSE,
        datamods::import_file_ui(
          title = "", ns("import-metadata-fcs"), preview_data = F,
          file_extensions = c(".csv", ".txt", ".xls", ".xlsx")
        ),
        withSpinner(DT::dataTableOutput(outputId = ns("data_metadata_fcs")), type=6),
        br(),
        fluidRow( ## fluidrow mfis ####
          column(
            width = 5,
            div(class = "InputSized", uiOutput(ns("sharedColMetaFcs_UI"))) # "id_col"  "id
          ),
          column(
            width = 2
           # , div(class = "InputSized", uiOutput(ns("fileformat_metadata_fcs"))) # file_format = c("wide", "long"),
          ),
          # column(
          #   width = 4,
          #   div(class = "InputSized", uiOutput(ns("fcs_ids_type_fcs_input"))) # patient , patient_cluster ...
          # ),
          column(
            width = 5,
            div(class = "InputSized", uiOutput(ns("column_name_fcs_concatened"))) # "...1"
          ),
          column(
            width = 8,
             uiOutput(ns("other_metadata_columns")) # c("condition", "batch")
          )
        )
      )
    ),
    fluidRow(
      box(
        width = 12, title = "create_sce Parameters",
        div(
          class = "magnificationDiv", style = "position:absolute; right: 0px; top: 0px;",
          actionBttn(
            inputId = "paramsFcsDisplay", style = "minimal",
            color = "danger", class = "magnifButton", label = icon("minus"), value = FALSE
          )
        ),
        conditionalPanel(
          condition = "input.paramsFcsDisplay %2 == 1",
          verbatimTextOutput(outputId = ns("params_fcs_object_see"))
        )
      )
    )
  )
}

#' import_concat Server Functions
#'
#' @importFrom flowCore pData read.flowSet parameters
#' @importFrom methods as
#' @noRd
mod_import_concat_server <- function(id, r_global, reloadTrigger) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns
    state_load_fcs <- reactiveVal(FALSE)

    observeEvent(input$fileUploadonServer, {
      file <- input$fileUploadonServer
      if (is.null(file)) {
        return()
      }

      # Define the base directory
      base_dir <- file.path(projects_folder(), r_global$project_folder_name)

      # Create the 'uploaded_fcs' subdirectory if it doesn't exist
      sub_dir <- file.path(base_dir, "uploaded_fcs")
      if (!dir.exists(sub_dir)) {
        dir.create(sub_dir, recursive = TRUE)
      }

      file_dest <- file.path(sub_dir, file$name)

      # Save the file
      file.rename(file$datapath, file_dest)


      # Optional: Update UI or server state
    })


    imported_metadata_fcs <- datamods::import_file_server(
      id = "import-metadata-fcs",
      return_class = "data.frame",
      trigger_return = "change"
    )

    datamods::import_file_server("import-metadata-fcs", trigger_return = "change")


    observe({
      how <- "success" %in% c(imported_metadata_fcs$status())

      state_load_fcs(how)
    })


    defaultValue <- reactiveValues(
      clustCol = "filter",
      markerCol = "ftr (sec)",
      mfiValueCol = "median_value",
      sharedCol = "file",
      totalCell = "Unfiltered|count",
      file_format_abund = "wide",
      file_format_mfi = "long",
      fcs_names = "patient",
      ppatient = 1,
      pcluster = 1,
      is_scaled = T,
      to_scaled = F,
      fcs_to_cut = F
    )


    shinyDirChoose(
      input,
      "FCSdir",
      roots = c(home = "~"),
      filetypes = c("", "fcs")
    )

    FCSpath <- reactiveValues(
      datapath = NULL,
      fileList = NULL,
      flowSet = NULL
    )

    # output$ref_pData_name <- renderUI({
    #   req(FCSpath$pData)
    #   
    #   pickerInput(
    #     inputId = ns("select_ref_pData"), "Marker name:", choices = colnames(FCSpath$pData),
    #     selected = NULL
    #   )
    #   
    # })

    observeEvent(
      ignoreNULL = TRUE,
      eventExpr = {
        input$FCSdir
      },
      handlerExpr = {
        if (!"path" %in% names(input$FCSdir)) {
          return()
        }
        home <- normalizePath("~")
        FCSpath$datapath <- file.path(home, paste(unlist(input$FCSdir$path[-1]), collapse = .Platform$file.sep))
        FCSpath$fileList <- list.files(FCSpath$datapath)
        FCSpath$flowFrame <- as(flowCore::read.flowSet(
          path = FCSpath$datapath,
          pattern = "\\.fcs$",
          transformation = NULL,
          truncate_max_range = FALSE,
          min.limit = NULL
        ), "flowFrame")
        FCSpath$pData <- flowCore::pData(flowCore::parameters(FCSpath$flowFrame))
        FCSpath$dataFrame <- FCSpath$flowFrame@exprs |> as.data.frame()
        print("done")
      }
    )

    output$pData <- DT::renderDataTable(
      {
        FCSpath$pData
      },
      options = list(
        scrollX = "400px",
        pageLength = -1
      )
    )

    selected_table_markers <- reactiveVal()

    observe({
      # selected_table_markers(unname(FCSpath$pData[input$pData_rows_selected, input$select_ref_pData]))
      selected_table_markers(unname(FCSpath$pData[input$pData_rows_selected, "name"]))
    })

    output$Label_Markers_table_selected <- renderText({
      paste(selected_table_markers(), collapse = ", ")
    })

    output$sharedColFcs_UI <- renderUI({
      req(FCSpath$dataFrame)
      pickerInput(
        inputId = ns("sharedColFCS"), "Shared column:", choices = colnames(FCSpath$dataFrame),
        selected = NULL
      )
    })


    output$Label_Clusters <- renderUI({
      req(FCSpath$pData)
      pickerInput(
        inputId = ns("ClusterList"),
        label = "Choose Clustering:",
        choices = list(
          # `without desc` = FCSpath$pData |> subset(is.na(desc)) |> dplyr::pull(.data$name) |> unname()
          `all` = FCSpath$pData |> dplyr::pull(.data$name) |> unname()
        ),
        options = list(
          `actions-box` = TRUE
        ),
        multiple = TRUE
      )
    })


    output$transformation_fcs <- renderUI({
      req(FCSpath$dataFrame)
      awesomeCheckbox(ns("Asinh_transformation"),
        label = HTML("<b>asinh transformation<b>"),
        value = defaultValue$to_scaled
      )
    })

    output$asinh_UI_cofact_fcs <- renderUI({
      req(FCSpath$dataFrame)

      numericInput(
        inputId = ns("fcs_cofact"),
        "Chose cofactor value:",
        value = 5, step = 1, min = 0
      )
    })



    output$createSCE_button <- renderUI({
      req(state_load_fcs())
      actionBttn(
        inputId = ns("runSCE"),
        class = "btn-secondary",
        icon = icon("gears"),
        style = "material-flat",
        label = "Create a cell object(3: sce)"
      )
    })



    output$sharedColMetaFcs_UI <- renderUI({
      req(imported_metadata_fcs$data())
      pickerInput(
        inputId = ns("sharedColFcsMeta"), "Shared column:", choices = colnames(imported_metadata_fcs$data()),
        selected = NULL
      )
    })



    # output$fileformat_metadata_fcs <- renderUI({
    #   req(imported_metadata_fcs$data())
    #   pickerInput(
    #     inputId = ns("format_metadata_fcs"), "Table Format:", choices = c("wide"),
    #     choicesOpt = list(
    #       icon = c("glyphicon-arrow-right")
    #     ),
    #     selected = "wide"
    #   )
    # })


    # output$fcs_ids_type_fcs_input <- renderUI({
    #   req(imported_metadata_fcs$data())
    #   pickerInput(ns("fcs_names_fcs_input"),
    #     label = "contained in fcs:",
    #     choices = list("patient", "cluster", "patient_cluster"),
    #     selected = defaultValue$fcs_names
    #   )
    # })


    output$column_name_fcs_concatened <- renderUI({
      req(imported_metadata_fcs$data())
      pickerInput(
        inputId = ns("concatened_col_fcs"), "fcs name column:",
        choices = c(colnames(imported_metadata_fcs$data())),
        selected = "...1"
      )
    })

    output$other_metadata_columns <- renderUI({
      req(imported_metadata_fcs$data())
      pickerInput(
        inputId = ns("other_metadata_col_fcs"),
        "interest metadata column:", choices = c(colnames(imported_metadata_fcs$data())),
        selected = c("condition", "batch"), multiple = T
      )
    })


subset_annot <- reactive({
  FCSpath$pData[input$pData_rows_selected,]
})


    param_list_fcs <- reactive({
      obj <- list(
        "annot_markers" = subset_annot(),
        "ff" = FCSpath$flowFrame, # flowFrame object
        "meta" = imported_metadata_fcs$data(), # recop_metadata
        "vector_markers" = selected_table_markers(), # all_col[c(3:24)]
        "vector_clustering" = input$ClusterList, # clustering
        "id" = input$sharedColFCS, # "fcsID"
        # "cell_id" = input$cellIDColFCS, # "Event #"
        "metadata_colnames" = list(
          "id" = input$sharedColFcsMeta,
          # "n_cells" = input$num_cells_col_fcs,
          "fcs_names" = input$concatened_col_fcs,
          "subset" = input$other_metadata_col_fcs
        ), # list(id="fcsID" ,n_cells="$TOT",fcs_names="...1",subset=c("condition", "batch"))
        "transform" = input$Asinh_transformation,
        "applied_fct" = "asinh",
        "cofactor" = input$fcs_cofact
      )

      return(obj)
    })


    observeEvent(input$runSCE, {
      if (is.null(param_list_fcs()[["vector_markers"]])) {
        showModal(
          modalDialog(
            title = "Error",
            p("Please select some markers.")
          )
        )
      } else if (is.null(param_list_fcs()[["vector_clustering"]])) {
        showModal(
          modalDialog(
            title = "Error",
            p("Please choose at leat a clustering."),
          )
        )
      }
    })

    observeEvent(input$runSCE, {
      req(param_list_fcs()[["vector_markers"]])
      req(param_list_fcs()[["vector_clustering"]])

      # if (!fcs_downloaded()) {
      r_global$sce_data <- analycyte.utils::create_sce(
        ff = param_list_fcs()[["ff"]],
        annot_markers = param_list_fcs()[["annot_markers"]],
        meta = param_list_fcs()[["meta"]],
        vector_markers = param_list_fcs()[["vector_markers"]],
        vector_clustering = param_list_fcs()[["vector_clustering"]],
        id = param_list_fcs()[["id"]],
        # cell_id = param_list_fcs()[["cell_id"]],
        metadata_colnames = param_list_fcs()[["metadata_colnames"]],
        transform = param_list_fcs()[["transform"]],
        applied_fct = param_list_fcs()[["applied_fct"]],
        cofactor = param_list_fcs()[["cofactor"]]
      )


      showNotification("SCE loaded", type = "message")
      reloadTrigger(reloadTrigger() + 1)

      new_step(project = r_global$project_id, label = "mod_import_concat", output_type = "sce")

      last_SID <- steps() |>
        dplyr::pull(.data$SID) |>
        max()

      step_row <- steps() |> dplyr::filter(.data$SID == last_SID)
      project_row <- projects(verbose = T) |> dplyr::filter(.data$id == step_row$PID)

      # recuperation du hash_tag

      hash <- step_row |> dplyr::pull(.data$hash_tag)
      project_path <- project_row |> dplyr::pull(.data$path)

      path <- unclass(fs::path(project_path, hash))

      # ecriture du data sur le disque

      if (dir.exists(path)) {
        saveRDS(r_global$sce_data, file = file.path(path, "dataSCE.rds"))
        showNotification("RDS saved", type = "message")
        reloadTrigger(reloadTrigger() + 1)
      } else {
        showNotification("No project hash folder found.", type = "warning")
        reloadTrigger(reloadTrigger() + 1)
      }

      finish_step(project = r_global$project_id, SID = last_SID)




      #   dataSCE(data)
      #   fcs_downloaded(TRUE)
      #   showNotification("fcs loaded", type = "message")
      # } else {
      #   showNotification("fcs already loaded", type = "message")
      # }
    })





    output$params_fcs_object_see <- renderPrint({
      req(param_list_fcs())
      print(param_list_fcs())
    })

    output$data_metadata_fcs <- DT::renderDataTable(
      imported_metadata_fcs$data(),
      escape = FALSE, 
      selection = "none", server = FALSE,rownames=FALSE,
      options = list(dom = "t", scrollX = TRUE, ordering = FALSE)
    )
  })
}

## To be copied in the UI
# mod_import_concat_ui("import_concat_1")

## To be copied in the server
# mod_import_concat_server("import_concat_1")
