#' import_rds UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#' @importFrom shiny NS tagList
mod_import_rds_ui <- function(id, r_global) {
  ns <- NS(id)
  tagList(
    div(
      id = id,
      fluidRow(
        box(
          width = 12, status = "primary",
          div(
            column(
              width = 6, strong("Project of Reference: "),
              span(style = "color: #007bff;", r_global$project_folder_name) # Styling for the text
            )
          )
        )
      ),
      fluidRow(
        column(
          width = 12,
          h2("RDS file import", class = "module-title-bold")
        )
      ),
      fluidRow(
        column(
          width = 5,
          fileInput(ns("file_rds"), "Load existing FE as RDS File",
            accept = c("rds")
          )
        ),
        box(
          width = 7,
          status = "warning", solidHeader = T,
          title = "Import a .rds file",
          markdown('
**Functionality Overview:**
This panel enables users to import an existing `.rds` file that corresponds to a previously processed Flow Cytometry Standard (FCS) file.

**Instructions:**
1. **Browse File:** Click on the \'Browse\' button to navigate through your local directory.
2. **Select File:** Look for the `.rds` file you wish to import. These files represent pre-processed FCS data.
3. **Confirmation:** Once the file is successfully uploaded, a confirmation message will be displayed.
5. **Next Steps:** After importing, you can proceed to new actions in the "Select an action" box.

**Additional Information:**
- This panel is designed to streamline the workflow by allowing easy access to pre-processed data.
')
        )
      )
    )
  )
}

#' import_rds Server Functions
#'
#' @noRd
mod_import_rds_server <- function(id, r_global, reloadTrigger) {
  moduleServer(id, function(input, output, session) {
    ns <- session$ns


    # Check if a file has been selected
    observeEvent(input$file_rds, {
      # Read the file if it's an RDS file
      if (endsWith(input$file_rds$name, ".rds")) {
        r_global$data <- readRDS(input$file_rds$datapath)

        # Check that the data is a list with the required components
        if (is.list(r_global$data) && all(c("SE_abundance") %in% names(r_global$data))) {
          showNotification("FE loaded", type = "message")
          reloadTrigger(reloadTrigger() + 1)

          new_step(project = r_global$project_id, label = "mod_import_rds", output_type = "fe")

          last_SID <- steps() |>
            dplyr::pull(.data$SID) |>
            max()

          step_row <- steps() |> dplyr::filter(.data$SID == last_SID)
          project_row <- projects(verbose = T) |> dplyr::filter(.data$id == step_row$PID)

          # recuperation du hash_tag

          hash <- step_row |> dplyr::pull(.data$hash_tag)
          project_path <- project_row |> dplyr::pull(.data$path)

          path <- unclass(fs::path(project_path, hash))

          # ecriture du data sur le disque

          if (dir.exists(path)) {
            saveRDS(r_global$data, file = file.path(path, "dataFE.rds"))
            showNotification("RDS saved", type = "message")
            reloadTrigger(reloadTrigger() + 1)
          } else {
            showNotification("No project hash folder found.", type = "warning")
            reloadTrigger(reloadTrigger() + 1)
          }

          finish_step(project = r_global$project_id, SID = last_SID)
        } else {
          # Display an error message if the data is not valid
          showModal(modalDialog("Error: Invalid RDS file. The file should be a list with 'FE' components.",
            footer = modalButton("OK")
          ))
        }
      } else {
        # Display an error message if the file is not an RDS file
        showModal(modalDialog("Error: The selected file is not an RDS file.",
          footer = modalButton("OK")
        ))
      }
    })
  })
}

## To be copied in the UI
# mod_import_rds_ui("import_rds_1")

## To be copied in the server
# mod_import_rds_server("import_rds_1")
