library(flowCore)
library(dplyr)
library(tidyr)


# ===== READ FILES =====

project = "immunopanc_jg"
fcs_file = "/home/lohmann/immunopanc_jg/11_concat_cytosplore/nk/11_concat_NK.fcs"
ann_file = "/home/lohmann/immunopanc_jg/11_concat_cytosplore/nk/FCS_METADATA.xlsx"

data_source = sprintf("Project: %s      File: %s", project, basename(fcs_file))

ff = read.FCS(fcs_file, transformation = NULL, truncate_max_range = FALSE, min.limit = NULL)

pData(parameters(ff))


ann = readxl::read_excel(ann_file)

ann




# ===== PROCESS FF =====

colnames(ff)
cat('c("', paste0(colnames(ff), collapse = '", "'), '")', sep ="")

ff_col_ann = pData(parameters(ff))
ff_col_ann$info = sub("^(.+?_)", "", ff_col_ann$desc)
ff_col_ann$info[is.na(ff_col_ann$desc)] =
    ff_col_ann$name[is.na(ff_col_ann$desc)]
colnames(ff) = ff_col_ann$info
colnames(ff)
cat('c("', paste0(colnames(ff), collapse = '", "'), '")', sep ="")

exprs(ff)[, "191Ir"] = asinh(exprs(ff)[, "191Ir"]/5)
exprs(ff)[, "193Ir"] = asinh(exprs(ff)[, "193Ir"]/5)


# ===== PROCESS ANN =====

colnames(ann)
cat('c("', paste0(colnames(ann), collapse = '", "'), '")', sep ="")

# filter columns
head(ann)
ann = ann[, c("patient_id", "time_id", "fcs_id", "filename", "expdate",
              "PDAC vs Benign", "N+ vs N- ADK", "POPF", "DPC vs PG", "Resection")]

# simplify column names
# col_rename = function(long, short) {
#     colnames(ann)[colnames(ann) == long] <<- short
# }
# col_rename("PDAC vs Benign (1 vs 0; vide= pas utilisable)", "PDAC vs Benign")
# col_rename("DPC vs PG (1=DPC; 0=PG; vide= pas utilisable)", "DPC vs PG")
# col_rename("Resection (1=oui)", "Resection")
# col_rename("N+ vs N- ADK (1 vs 0; vide= pas utilisable)", "N+ vs N- ADK")
# colnames(ann)

# rename date
ann$expdate
#ann$expdate = gsub("211122", "211222", ann$expdate)
ann$expdate = gsub("221011", "221110", ann$expdate)

# build discrete variables
ann$expdate = factor(ann$expdate)
ann$ip = sub("(^..).+", "\\1", ann$patient_id)
ann$date_ip = with(ann, factor(
    paste(expdate, ip),
    levels = Reduce(paste, expand.grid(unique(ip), unique(expdate))[,2:1])))
table(ann$date_ip)
ann$ip_date = with(ann, factor(
    paste(ip, expdate),
    levels = Reduce(paste, expand.grid(unique(expdate), unique(ip))[,2:1])))
table(ann$ip_date)


# ===== ASSEMBLE DATA =====

dt = merge(flowCore::exprs(ff), ann)
colnames(dt)
dim(ff)
dim(dt)

sapply(dt, class)
sapply(dt, typeof)
table(dt$fcs_id)
summary(dt$CD16)
table(dt$ip_date)
table(dt$date_ip)

# ===== COUNTS =====

channels = c("CD16", "BTN3", "ICT05-222", "BTN2", "Tbet", "NKp46", "CD158ah", "NKG2D", "CD33", "CD107a", "TCR_pangd", "CD27", "BclXl", "CD96", "NKG2A", "CD3", "191Ir", "193Ir", "CD19", "CD45RA", "CD4", "CD8", "Bcl2", "TCRVd2", "NKG2C", "Eomes", "Siglec7", "CD158b1b2j", "Ki67", "NKp30", "CD45", "DNAM-1", "CD57", "Granzyme_B", "anti_IgG4", "CD56")


channels_reduced = setdiff(channels, c("CD19", "CD3", "ICT05-222"))

# extract counts per cluster using dplyr
counts = dt[, c("cluster_id", "fcs_id", channels_reduced)] |>
    group_by(cluster_id, fcs_id) |>
    summarise(n = n(), .groups = "drop") |>
    pivot_wider(names_from = fcs_id, values_from = n) |>
    select(!cluster_id)


# ===== MFIS =======

# extract mfis
mfis = dt[, c("cluster_id", "fcs_id", channels_reduced)] |>
    group_by(cluster_id, fcs_id) |>
    summarise(across(everything(), median), .groups = "drop")
cols = setdiff(colnames(mfis), c("cluster_id", "fcs_id"))
mfis = lapply(
    cols,
    function(x) {
        mfis |> select(all_of(c("cluster_id", "fcs_id", x))) |>
            pivot_wider(names_from = fcs_id, values_from = x) |>
            select(!cluster_id)
    })
names(mfis) = cols


# ===== SUBSET =====
#

question = 'N+ vs N- ADK'
question = 'DPC vs PG'
fcs_sel = which(!is.na(ann[question]))
fcs_sel = fcs_sel[order(ann[fcs_sel, question, drop = TRUE], fcs_sel)]

# subset
counts_sel = counts[,fcs_sel]
mfis_sel = lapply(mfis, function(x) x[,fcs_sel])
ann_sel = ann[fcs_sel,]


# heatmap diff
library(tibble)  # for column_to_rownames

# median
cluster_median = aggregate(dt[, channels_reduced], by = list(dt$cluster_id), median)[,-1]
lvls = sort(unique(ann_sel[,question, drop = TRUE]))
l = lvls[1]
question

ann[[question]] = factor(ann[[question]], levels = c(0,1), labels = c("DPC", "PG"))
lvls = as.list(levels(ann[[question]]))
names(lvls) = unlist(lvls)
# lvls = list("DPC" = 0, "PG" = 1)
lvls$all = unlist(lvls)

mfis_q = lapply(lvls, function(l) {
    fcs_id_sel_lvl = fcs_sel[ann[fcs_sel, question, drop = TRUE] %in% l]
    filter(dt, fcs_id %in% fcs_id_sel_lvl) |>
        select(all_of(c("cluster_id", channels_reduced))) |>
        group_by(cluster_id) |>
        summarise(across(everything(), median), .groups = "drop") |>
        column_to_rownames("cluster_id")
})

# dev copy
dev.copy.png <- function(file, width = 700, height = 700) {
    dev.copy(png, file = sprintf("%s.png", file),width = width, height = height)
    dev.off()
}

# normalize MFI
norm_cluster_mfi <- function(x, ref, type = c("pos", "diff"), probs = c(0.05, 0.99)) {
    if (missing(ref)) ref = x
    type = match.arg(type)
    # compute the scaling limits
    quant = Map(function(v) {
        q = quantile(v, probs = probs, type = 6)
    }, ref)
    # scale according to limits
    normd = Map(function(n) {
        q = quant[[n]]
        dq = q[2]-q[1]
        if (dq == 0) {
            rep(0, length(x[[n]]))
        } else if (type == "pos") {
            (x[[n]] - q[1]) / dq
        } else {
            x[[n]] / dq
        }
    }, names(quant))
    # return a data.frame
    data.frame(normd, check.names = FALSE)
}

# Eugenie
mfi_colors = grDevices::colorRampPalette(c(rev(RColorBrewer::brewer.pal(6, "Blues")), RColorBrewer::brewer.pal(6, "OrRd")))(100)
# More white at center
mfi_colors = grDevices::colorRampPalette(c(rev(RColorBrewer::brewer.pal(6, "Blues")), "#F5F1EE", RColorBrewer::brewer.pal(6, "OrRd")))(100)

# full dataset
res = pheatmap::pheatmap(
    norm_cluster_mfi(cluster_median),
    color = mfi_colors,
    breaks = seq(0,1, length.out = 101), main = "full experiment")
dev.copy.png("question_1_full_set")

# dataset of the question
pheatmap::pheatmap(
    norm_cluster_mfi(mfis_q$all, cluster_median),
    cluster_rows = res$tree_row, cluster_cols = res$tree_col,
    color = mfi_colors,
    breaks = seq(0,1, length.out = 101), main = "all samples")
dev.copy.png("question_2_set")

# classical heatmap
pheatmap::pheatmap(
    norm_cluster_mfi(mfis_q[[lvls[[1]]]], cluster_median),
    cluster_rows = res$tree_row, cluster_cols = res$tree_col,
    color = mfi_colors,
    breaks = seq(0,1, length.out = 101), main = sprintf("%s", lvls[1]))
dev.copy.png(sprintf("question_3_%s", lvls[[1]]))
pheatmap::pheatmap(
    norm_cluster_mfi(mfis_q[[lvls[[2]]]], cluster_median),
    cluster_rows = res$tree_row, cluster_cols = res$tree_col,
    color = mfi_colors,
    breaks = seq(0,1, length.out = 101), main = sprintf("%s", lvls[2]))
dev.copy.png(sprintf("question_3_%s", lvls[[2]]))

# differential heatmap
dd = norm_cluster_mfi(mfis_q[[lvls[[1]]]]-mfis_q[["all"]], cluster_median, "diff")
pheatmap::pheatmap(
    dd,
    cluster_rows = res$tree_row, cluster_cols = res$tree_col,
    color = mfi_colors,
    breaks = seq(-.5,.5, length.out = 101), main = sprintf("%s - all", lvls[1]))
dev.copy.png(sprintf("question_5_%s-all", lvls[[1]]))

dd = norm_cluster_mfi(mfis_q[[lvls[[2]]]]-mfis_q[["all"]], cluster_median, "diff")
pheatmap::pheatmap(
    dd,
    cluster_rows = res$tree_row, cluster_cols = res$tree_col,
    color = mfi_colors,
    breaks = seq(-.5,.5, length.out = 101), main = sprintf("%s - all", lvls[2]))
dev.copy.png(sprintf("question_5_%s-all", lvls[[2]]))

# between groups heatmap
dd = norm_cluster_mfi(mfis_q[[lvls[[2]]]]-mfis_q[[lvls[[1]]]], cluster_median, "diff")
pheatmap::pheatmap(
    dd,
    cluster_rows = res$tree_row, cluster_cols = res$tree_col,
    color = mfi_colors,
    breaks = seq(-.5,.5, length.out = 101), main = sprintf("%s - %s", lvls[2], lvls[1]))
dev.copy.png(sprintf("question_6_%s-%s", lvls[[2]], lvls[[1]]))
